/* ext - file extension stuff
 *
 *      (c)2007 Scott Lawrence
 *	   yorgle@gmail.com
 */

/*
 * Copyright (C) 2007 Scott Lawrence
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <stdio.h>	/* NULL */
#include <string.h>	/* strlen */
#include <strings.h>	/* strcasecmp */
#include <sys/types.h>  /* stat */
#include <sys/stat.h>   /* stat */

/* _is_dir
 *	returns 1 if it's a directory, 0 if it's not
 */
int mikmodule_is_dir(const char *filename)
{
	struct stat st;
	if (stat(filename, &st) < 0) {
		perror(filename);
		return -1;
	}
	return S_ISDIR(st.st_mode);
}



/* this extension list was taken from the Mikmod source */
static char * module_ext[] = {
	"669", "amf", "asy", "dsm", "far",
	"gdm", "imf", "it",  "m15", "med",
	"mod", "mtm", "okt", "s3m", "stm",
	"stx", "ult", "uni", "xm",
	NULL
};


/* MikMod playlist */
static char * playlist_ext[] = {
	"mik", "txt", "pls", "m3u",
	NULL
};

/* "tight" playlists */
static char * playlist_ext_tight[] = {
	"mik", "pls",
	NULL
};


/* _ext_search
 *	search through a NULL terminated "list", if "filename" matches,
 *	then return a 1, otterwise, return 0
 */
int mikmodule_ext_search( char ** list, const char * filename )
{
	int p = 0;

        if( !filename )
                return( 0 );

        if( strlen( filename ) < 4 )
                return( 0 );

	if( !list ) 
		return( 0 );

	/* go through our extensions list above, to determine if 
           the file can be opened through us.  :D */
	for( p=0 ; list[p] != NULL ; p++ )
	{
		if( !strcasecmp( list[p], filename + strlen( filename ) 
					           - strlen( list[p] ) )) {
			return( 1 );
		}
	}

        return( 0 );
}


/* _ext_openable
 *	returns 1 if the filename is a known type for a song file
 */
int mikmodule_ext_openable( const char * filename )
{
	if( mikmodule_is_dir( filename ) ) return( 0 );
	return( mikmodule_ext_search( module_ext, filename ));
}       


/* _ext_playlistable
 *	returns 1 if the filename is a known type for a playlist file
 */
int mikmodule_ext_playlistable( const char * filename )
{
	if( mikmodule_is_dir( filename ) ) return( 1 );
	return( mikmodule_ext_search( playlist_ext, filename ));
}

/* _ext_tight_playlist
 *	returns 1 if the filename is .pls or .mik 
 */
int mikmodule_ext_tight_playlist( const char * filename )
{
	// by commenting out the next line (adding -1 lines of code), we 
	// get our symlink-folder SymPlaylists
	//if( mikmodule_is_dir( filename ) ) return( 0 );
	// we should protect against false negatives though;
	// make sure it's an actual file, or a symlink
	return( mikmodule_ext_search( playlist_ext_tight, filename ));
}
