/* MikModule
 *
 *      interface to MikMod for podzilla 2
 *      
 *      (c)2007 Scott Lawrence
 *	   yorgle@gmail.com

----------------------------------------------------------------------------------------------------
NOTE: you probably want to have your terminal be 100 characters wide.

 */

/*
 * Copyright (C) 2007 Scott Lawrence
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/* please see version.h for the changelog and such */

#include <unistd.h>
#include <string.h>
#include <ctype.h>	/* isspace */
#include <time.h>	/* for randomize timer */
#include "ttk.h"	/* TTK_EV_UNUSED */
#include "pz.h"
#include "mikmod.h"
#include "playlist.h"
#include "mikmodule.h"
#include "render.h"
#include "ext.h"
#include "scan.h"
#include "version.h"


/* we use some pz.h constants that were put in recently.  sorry. */
#ifndef SHUFFLE_OFF
#define SHUFFLE_OFF     (0)
#define SHUFFLE_SONGS   (1)
#define SHUFFLE_ALBUMS  (2)     /* not implemented yet. */
#define REPEAT_OFF      (0)
#define REPEAT_ONE      (1)
#define REPEAT_ALL      (2)
//#error *** You need to update your podzilla
//#error ***  source to 2007-Jan-12 (r1401)
//#error ***  or newer!!
#endif


/* options used for the settings */
const char * noyes_options[] = { "No", "Yes", 0 };
const char * yesno_options[] = { "Yes", "No", 0 };
const char * buffer_options[] = { "1", "2", "4", "8", "16", 0 };
const char * highlow_options[] = { "High", "Med", "Low", "Mono", 0 };
const char * recurse_options[] = { "Never", "Always", "Ask", 0 };
const char * gain_options[] = { "1x", "2x", "4x", 0 };

/* declaration of our globals */
MYGLOBALS mikGlob;


/* start up a popup overlay on the screen */
void mikmodule_newOverlay( char * buf, int ticks )
{
	mikGlob.overlayEnd = mikGlob.ticks+ticks;
	snprintf( mikGlob.overlay, 128, "%s", (buf)?buf:"-" );
}


/* _timedMessage
 *	display a message, but let it time out
 */
void mikmodule_timedMessage( char * title, char * body )
{
	if( pz_get_int_setting( mikGlob.config, CFG_AUTODISMISS )) {
		pz_do_dialog( title, body, _("Ok"), 0, 0, 0, 0 );
	} else {
		pz_do_dialog( title, body, _("Ok"), 0, 0, 2, 0 );
	}
}


/* called when pz2 shuts down */
void mikmodule_destroy( void )
{
	pz_save_config( mikGlob.config );
}

/* initialize the playback system */
void mikmodule_startup( void )
{
	/* initialize the library */
	md_mode |= DMODE_SOFT_MUSIC;
	if (MikMod_Init( "" )) {
		mikGlob.startedup = 0;
	    	fprintf(stderr, "Could not initialize sound, reason: %s\n",
			MikMod_strerror(MikMod_errno));
	} else {
		mikGlob.consecutiveErrors = 0;
		mikGlob.startedup = 1;
		mikGlob.editmode = EDIT_IDLE;
		mikGlob.viewmode = pz_get_int_setting( mikGlob.config, CFG_VIEWMODE );
		mikGlob.playlist = NULL;
		mikGlob.nTracks = 0;
		mikGlob.scrubButton = 0;
		mikGlob.nowPlaying = NULL;
		mikGlob.volume = pz_get_int_setting( mikGlob.config, CFG_VOLUME );
		md_volume = mikGlob.volume;
	}
}


/* stop playback and close things up for the session */
void mikmodule_shutdown( void )
{
	MikMod_DisableOutput();
	if( mikGlob.startedup ) {
		mikGlob.startedup = 0;
		Player_Stop();
		playlist_delete_list( mikGlob.playlist );
		mikGlob.playlist = NULL;
		mikGlob.nowPlaying = NULL;
	}

	/* give up */
	MikMod_Exit();
}


#ifdef NEVER
/* just dumps out current song info to stdout */
static void mikmodule_dumpInfo( void )
{
	int i;
	long engineversion = MikMod_GetVersion();

	printf("MikMod library version: %ld.%ld.%ld\n",
               (engineversion >> 16) & 255,
               (engineversion >> 8) & 255,
               (engineversion) & 255 );

	if( !mikGlob.nowPlaying ) return;
	if( !mikGlob.nowPlaying->song ) return;

	printf( "Module:\n" );
	if( mikGlob.nowPlaying->song->songname ) {
		printf( "    Title: %s\n", mikGlob.nowPlaying->song->songname );
	}
	if( mikGlob.nowPlaying->song->comment ) {
		printf( "    comment: %s\n", mikGlob.nowPlaying->song->comment );
	}
	if( mikGlob.nowPlaying->song->modtype ) {
		printf( "    modtype: %s\n", mikGlob.nowPlaying->song->modtype );
	}
	printf( "     numchn: %d\n", mikGlob.nowPlaying->song->numchn );
	printf( "  numvoices: %d\n", mikGlob.nowPlaying->song->numvoices );
	printf( "     numpos: %d\n", mikGlob.nowPlaying->song->numpos );
	printf( "     numpat: %d\n", mikGlob.nowPlaying->song->numpat );
	printf( "     numins: %d\n", mikGlob.nowPlaying->song->numins );
	printf( "     numsmp: %d\n", mikGlob.nowPlaying->song->numsmp );
	printf( "    sngtime: %ld\n", mikGlob.nowPlaying->song->sngtime );
	printf( "     sngspd: %d\n", mikGlob.nowPlaying->song->sngspd );
	printf( "        bpm: %d\n", mikGlob.nowPlaying->song->bpm );
	printf( "    realchn: %d\n", mikGlob.nowPlaying->song->realchn );
	printf( "   totalchn: %d\n", mikGlob.nowPlaying->song->totalchn );
	printf( "\n" );
	printf( "    panning:\n" );
	for( i=0 ; i<mikGlob.nowPlaying->song->numchn ; i++ ) {
		printf( " %d", mikGlob.nowPlaying->song->panning[ i ] );
	}
	printf( "\n" );
	printf( "    panflag: %d\n", mikGlob.nowPlaying->song->panflag );
	printf( "       wrap: %d\n", mikGlob.nowPlaying->song->wrap );
	printf( "       loop: %d\n", mikGlob.nowPlaying->song->loop );
	printf( "    fadeout: %d\n", mikGlob.nowPlaying->song->fadeout );
	printf( "\n" );
	printf( "     patpos: %d\n", mikGlob.nowPlaying->song->patpos );
	printf( "     sngpos: %d\n", mikGlob.nowPlaying->song->sngpos );
	printf( "     relspd: %d\n", mikGlob.nowPlaying->song->relspd );
}
#endif



/* advance to the next/previous track, or the first track if there was no track
** direction:	-1 is previous, 1 is next
*/
int mikmodule_trackAdvance( int direction )
{
	int verbosity = pz_get_int_setting( pz_global_config, VERBOSITY );

	if( !mikGlob.nowPlaying )
	{
		mikGlob.nowPlaying = mikGlob.playlist;
	} else {
		/* stop the song, free the ram */
		mikGlob.nowPlaying->playcount++;
		Player_Stop();
		Player_Free( mikGlob.nowPlaying->song );
		mikGlob.nowPlaying->song = NULL;

		if( direction > 0 )
			mikGlob.nowPlaying = mikGlob.nowPlaying->next;
		else
			mikGlob.nowPlaying = mikGlob.nowPlaying->prev;
	}
	if( !mikGlob.nowPlaying ) return 0;

	/* now cache, load, and start it! */
	mikGlob.nowPlaying->song = Player_Load( mikGlob.nowPlaying->filename, 64, 0 );

	if( !mikGlob.nowPlaying->song ) {
		mikGlob.consecutiveErrors++;
		if( mikGlob.consecutiveErrors >= 8 ) {
			return( 0 );
		}
		if( verbosity < 2 ) 
		{
			char * fn = "-";
			int len = strlen( MikMod_strerror(MikMod_errno) );
			
			if( mikGlob.nowPlaying->filename )
				fn = mikGlob.nowPlaying->filename;

			len += strlen( fn );

			char * buf = malloc( len * sizeof( char ));
			snprintf( buf, len, "%s: %s", fn?fn:"?",
					MikMod_strerror(MikMod_errno) );

			// display the correct track name in the title
			ttk_window_title( mikGlob.window, fn );
			mikmodule_timedMessage( "MikMod Load Error", buf );
			free( buf );
		}
		return( mikmodule_trackAdvance( direction )); /* bad file fix? */
	} else {
		mikGlob.consecutiveErrors = 0;
	}

	return 1;
}



/* erase any whitespace (for song title display) */
char *zot_space( char *str )
{
	int d,s;
	
	/* trim the end */
	for( d = strlen( str )-1 ; isspace( str[d] ) && d>0 ; d-- )
		str[d] = '\0';

	/* trim the beginning, and shift */
	for( s = 0; isspace( str[s] ) && s < strlen( str ) ; s++ );

	for( d = 0; str[s] != '\0' ; s++, d++ )
	{
		str[d] = str[s];
	}
	str[d] = '\0';

	return( str );
}


static char * mikmodule_toggleWindowTitleInfo( void )
{
	char buf[80];
	char * tn;
	char * tt = "N/A";

	if (!mikGlob.nowPlaying || !mikGlob.nowPlaying->song) return NULL;

	/* make sure we have something to display, song name or short name */
	tn = zot_space( mikGlob.nowPlaying->song->songname );
	
	if( !tn || !strlen( tn )  ) {
		tn = mikGlob.nowPlaying->shortname;
	}

	if( mikGlob.whichHeader > MTW_MAX ) mikGlob.whichHeader = 0;

	switch( mikGlob.whichHeader ) {
	case( MTW_FILENAME ):
		if( mikGlob.nTracks > 1 ) {
			/* playlist */
			snprintf( buf, 80, "[%d/%d] %s", 
				mikGlob.nowPlaying->index, mikGlob.nTracks, 
				mikGlob.nowPlaying->shortname );
			tt = buf;
		} else {
			/* song name */
			tt = mikGlob.nowPlaying->shortname;
		}
		break;
	case( MTW_SONGPOS ):
		snprintf( buf, 80, "%03d/%03d  %02X  %d%%",
                        mikGlob.nowPlaying->song->sngpos,
                        mikGlob.nowPlaying->song->numpos,
                        mikGlob.nowPlaying->song->patpos,
                        (mikGlob.nowPlaying->song->sngpos * 100 / mikGlob.nowPlaying->song->numpos)
			 );
		tt = buf;
		break;
	case( MTW_SONGNAME ):
		tt = tn;
		break;
	}

	/* and shove it in the title bar */
	ttk_window_title( mikGlob.window, tt );

	return( tn );
}

/* restart the header info cycling at the filename/playlist position */
void mikmodule_restartHeaderCycle()
{
	mikGlob.whichHeader = MTW_FILENAME;
	mikGlob.headerTimeout = MTW_TIMEOUT_TICKS;
}

/* start the currently queued track */
void mikmodule_startTrack( void )
{
	int separation;
	if (!mikGlob.nowPlaying || !mikGlob.nowPlaying->song) return;

	/* now we setup the menu-settings */
	md_volume = mikGlob.volume;

	mikmodule_restartHeaderCycle();
	mikmodule_newOverlay( mikmodule_toggleWindowTitleInfo(), 40 );
	mikmodule_clearMikRender( &mikGlob );

	/* might want to use this later.. i dunno.  0=off, 15 = max, 1 = ok */
	md_reverb = 0;

	/* set the pan-pullin if necessary */
	separation = pz_get_int_setting( mikGlob.config, CFG_SEPARATION );
	if( separation ) {
		/* maybe move this into a slider, only just 4 channel mods */
		if( mikGlob.nowPlaying->song->numchn <= 4 ) {
			switch( separation ) {
			case( 0 ):
				md_pansep = 128; /* 128 = full / high */
				break;
			case( 1 ):
				md_pansep = 100; 
				break;
			case( 2 ):
				md_pansep = 64; 
				break;
			case( 3 ):
				md_pansep = 0;	/* 0 = mono / none */
			}
		}
	}
	mikGlob.nowPlaying->song->fadeout 
			= pz_get_int_setting( mikGlob.config, CFG_FADEOUT );
	mikGlob.nowPlaying->song->loop    
			= pz_get_int_setting( mikGlob.config, CFG_LOOPS );
	mikGlob.nowPlaying->song->panflag 
			= !pz_get_int_setting( mikGlob.config, CFG_PANNING );
	mikGlob.nowPlaying->song->extspd  
			= !pz_get_int_setting( mikGlob.config, CFG_PROTRACKER );

	/* heuristics checks */
	if( !pz_get_int_setting( mikGlob.config, CFG_HEURISTICS )){

		/* heuristic 1: testlast (9 fingers) - disable panning */
		if( mikGlob.nowPlaying->song->songname ) {
			if( !strcmp( "testlast", 
				     mikGlob.nowPlaying->song->songname)) {
				mikGlob.nowPlaying->song->panflag = 0;
			}
		}
	}

	/* start song playback */
	Player_Start( mikGlob.nowPlaying->song );

	/* and hide the volume or scrubber */
	mikGlob.editmode = EDIT_IDLE;
}



/* _scrub_timercheck
**	check to see if we should be scrubbing
*/
void mikmodule_scrub_timercheck( void ) 
{
	if( mikGlob.scrubTrigger ) {
		if( mikGlob.scrubButton == PZ_BUTTON_NEXT ) {
			mikGlob.editmode = EDIT_SCRUB;
			mikGlob.volscrubEnd = mikGlob.ticks + 10;
			Player_NextPosition();
		} else if( mikGlob.scrubButton == PZ_BUTTON_PREVIOUS ) {
			mikGlob.editmode = EDIT_SCRUB;
			mikGlob.volscrubEnd = mikGlob.ticks + 10;
			Player_PrevPosition();
		}
	}
}


/* handle a PZ/TTK event */
static int mikmodule_event( PzEvent *ev )
{
	int repeat = pz_get_int_setting( pz_global_config, REPEAT );

	switch( ev->type ) {
	case PZ_EVENT_SCROLL:
                if( !mikGlob.paused ) {
			/* extend the timeout */
			mikGlob.volscrubEnd = mikGlob.ticks + 100;
			switch( mikGlob.editmode ) {
			case( EDIT_IDLE ):
				TTK_SCROLLMOD( ev->arg, 10 ); /* eat a bunch of motion */
				if( ev->arg != 0 ) /* thanks DerPapst */
					mikGlob.editmode = EDIT_VOLUME;
				break;

			case( EDIT_VOLUME ):
				TTK_SCROLLMOD( ev->arg, 5 );
				if( ev->arg > 0 ) {
#define FIVEFIVELEVEL	(24)	/* below this gets single-step volume */
					/* XXX HACK FOR 5.5 VOLUME */
					if( mikGlob.volume < FIVEFIVELEVEL ) 
					    mikGlob.volume++;
					else
					    mikGlob.volume += 5;
					if( mikGlob.volume > 128 )
						mikGlob.volume = 128;
				} else {
					/* XXX HACK FOR 5.5 VOLUME */
					if( mikGlob.volume < FIVEFIVELEVEL ) 
					    mikGlob.volume--;
					else
					    mikGlob.volume -= 5;
					if( mikGlob.volume < 0 )
						mikGlob.volume = 0;
				}
				md_volume = mikGlob.volume;
				pz_set_int_setting( mikGlob.config,
						CFG_VOLUME, mikGlob.volume );
				break;

			case( EDIT_SCRUB ):
				TTK_SCROLLMOD( ev->arg, 10 );
				if( ev->arg > 0 ) {
					Player_NextPosition(); /* RIGHT */
				} else {
					Player_PrevPosition(); /* LEFT */
				}
				break;

			case( EDIT_VIEW ):
				TTK_SCROLLMOD( ev->arg, 10 );
				if( ev->arg > 0 ) {
					mikmodule_nextView();
				} else {
					mikmodule_prevView();
				}
				pz_set_int_setting( mikGlob.config, 
						CFG_VIEWMODE, mikGlob.viewmode ); 
				break;
			}
                        ev->wid->dirty = 1;
                }
                break;

        case PZ_EVENT_BUTTON_DOWN:
                switch( ev->arg ) {
                case( PZ_BUTTON_HOLD ):
                        mikGlob.paused = 1;
                        ev->wid->dirty = 1;
                        break;

		case( PZ_BUTTON_NEXT ):
			mikGlob.scrubButton = PZ_BUTTON_NEXT;
			mikGlob.scrubTrigger = 0;
			break;

		case( PZ_BUTTON_PREVIOUS ):
			mikGlob.scrubButton = PZ_BUTTON_PREVIOUS;
			mikGlob.scrubTrigger = 0;
			break;
                }
                break;

        case PZ_EVENT_BUTTON_HELD:
		switch( ev->arg ) {
		case( PZ_BUTTON_NEXT ):
		case( PZ_BUTTON_PREVIOUS ):
			mikGlob.scrubTrigger = 1;
			break;
		}
		return TTK_EV_UNUSED;

        case PZ_EVENT_BUTTON_UP:
                switch( ev->arg ) {
                case( PZ_BUTTON_MENU ):
			mikmodule_shutdown();
			pz_close_window (ev->wid->win);
                        break;

                case( PZ_BUTTON_HOLD ):
                        mikGlob.paused = 0;
                        ev->wid->dirty = 1;
                        break;

                case( PZ_BUTTON_PREVIOUS ):
			if( !mikGlob.scrubTrigger ) {
				if( mikmodule_trackAdvance( -1 )) {
					mikmodule_startTrack();
				} else {
					/* thanks to DerPapst for this idea */
					mikmodule_shutdown();
					pz_close_window (ev->wid->win);
					mikmodule_timedMessage( "MikModule", "Too many errors. Exiting." );
					return( 0 );
				}
				if( ev && ev->wid ) ev->wid->dirty = 1;
			}
			mikGlob.scrubTrigger = mikGlob.scrubButton = 0;
                        break;

                case( PZ_BUTTON_NEXT ):
			if( !mikGlob.scrubTrigger ) {
				if( mikmodule_trackAdvance( 1 )) {
					mikmodule_startTrack();
				}
                        	ev->wid->dirty = 1;
			}
			mikGlob.scrubTrigger = mikGlob.scrubButton = 0;
                        break;

                case( PZ_BUTTON_PLAY ):
			Player_TogglePause();
                        ev->wid->dirty = 1;
                        break;

                case( PZ_BUTTON_ACTION ):
			/* advance to the next mode, cycle around to vis vol */
			mikGlob.editmode++;
			mikGlob.volscrubEnd = mikGlob.ticks + 100;
			if( mikGlob.editmode > EDIT_MAX )
				mikGlob.editmode = EDIT_FIRST +1;
                        ev->wid->dirty = 1;
                        break;
		}
		break;

        case PZ_EVENT_DESTROY:
		mikmodule_shutdown();
                mikmodule_destroy();
                break;

        case PZ_EVENT_TIMER:
		/* after the timeout, switch to invisible volume */
		mikGlob.ticks++;
		if( mikGlob.ticks > mikGlob.volscrubEnd ) {
			mikGlob.editmode = EDIT_IDLE;
			mikGlob.volscrubEnd = 0;	/* turn off the overlay */
		}

		/* toggle our menu titlebar text */
		if( (mikGlob.whichHeader == MTW_SONGPOS)
			&& mikGlob.headerTimeout & 0x03 )
		{
			mikmodule_toggleWindowTitleInfo();
		}

		if( --mikGlob.headerTimeout <= 0 ) {
			mikGlob.whichHeader++;
			mikmodule_toggleWindowTitleInfo();
			mikGlob.headerTimeout = MTW_TIMEOUT_TICKS;
		}

		/* __TC__ */
		if(  mikGlob.ticks > mikGlob.updateTicks ) {
			mikGlob.updateTicks = mikGlob.ticks + 1; /* let one draw poll happen */
			/* if we're running... */
			if( mikGlob.startedup ) {
				if( Player_Active() ) {
					mikmodule_scrub_timercheck();

					/* poll the mikmod engine */
					MikMod_Update();
				} else {
					/* no mikmod; go to the next track or exit */
					if( mikmodule_trackAdvance( 1 ) ) {
						mikmodule_startTrack();
					} else {
						/* no next track, so exit or repeat */
						if( repeat == REPEAT_OFF ) {
							/* exit! */
							mikmodule_shutdown();
							pz_close_window (ev->wid->win);
						} else {
							/* repeat! */
							mikGlob.nowPlaying = NULL;
							mikmodule_trackAdvance( 1 );
							mikmodule_startTrack();
						}
					}
				}
			}
		}
		if( ev && ev->wid ) ev->wid->dirty = 1;
                break;
        }

	return( 0 );
}



/* mix 7 parts BG, 1 part FG
** this is used for various rendering
*/
static void mikmod_mix_bg2( void )
{
	int bg_r, bg_g, bg_b;
	int fg_r, fg_g, fg_b;

	ttk_unmakecol( mikGlob.fg, &fg_r, &fg_g, &fg_b );
	ttk_unmakecol( mikGlob.bg, &bg_r, &bg_g, &bg_b );

	mikGlob.bg2 = ttk_makecol(  (fg_r + bg_r*7)>>3,
				    (fg_g + bg_g*7)>>3,
				    (fg_b + bg_b*7)>>3 );
}

/* _mix_colors
 *	set up all of our important colors
 */
void mikmodule_mix_colors( void )
{
	int c;
	int r, g, b;

	/* setup bg2 */
	mikGlob.fg = ttk_ap_getx( "window.fg" )->color;
	mikGlob.bg = ttk_ap_getx( "window.bg" )->color;
	mikGlob.rd = ttk_ap_getx( "window.border" )->color;
	mikmod_mix_bg2();

	/* set up the colors */
	mikGlob.cols[ 0] = ttk_makecol( 255,   0,   0 ); /* red */
	mikGlob.cols[ 1] = ttk_makecol( 255, 255,   0 ); /* yellow */
	mikGlob.cols[ 2] = ttk_makecol(   0, 255,   0 ); /* green */
	mikGlob.cols[ 3] = ttk_makecol(   0, 255, 255 ); /* cyan */
	mikGlob.cols[ 4] = ttk_makecol(   0,   0, 255 ); /* blue */
	mikGlob.cols[ 5] = ttk_makecol( 200,   0, 200 ); /* purple */

        /* RANDOMIZE $TIMER */
        srand(time(NULL));

	/* the rest are random */
	for( c=6 ; c<32 ; c++ ) {
		int w = rand();
		r = (w&1)?(rand() & 0xe0):200;
		g = (w&1)?(rand() & 0xff):255;
		b = (w&1)?(rand() & 0xff):255;
		if( r<128 && g<128 && b<128 ) {
			r += 128;
			g += 128;
			b += 128;
		}
		mikGlob.cols[c] = ttk_makecol( r, g, b ); 
	}

	/* make the halfbrite version */
	for( c=0 ; c<32 ; c++ ) {
		ttk_unmakecol( mikGlob.cols[c], &r, &g, &b );
		mikGlob.colsHB[c] = ttk_makecol( r>>1, g>>1, b>>1 );
	}
}


/* create a new window and start the playlist passed in 
*/
PzWindow *mikmodule_new_window_with_playlist( PLAYLIST_ITEM * plst )
{
	/* set up the globals */
	mikGlob.startedup  = 0;
	mikGlob.module = NULL;
	mikGlob.window = NULL;
	mikGlob.widget = NULL;
	mikGlob.paused = 0;
	mikGlob.ticks  = 0l;
	mikGlob.updateTicks = 0l;
	mikGlob.overlay[0] = '\0';
	mikGlob.overlayEnd = 0l;
	mikGlob.nowPlaying = NULL;

	mikmodule_mix_colors();

	if( !plst ) {
		if( mikGlob.fromFile == 1 ) {
			mikmodule_timedMessage( "MikMod Load Error", "Invalid or empty playlist!" );
		} else {
			mikmodule_timedMessage( "MikMod Load Error", "No playable songs in directory!" );
			if( pz_get_int_setting( mikGlob.config, CFG_RECURSE ) != 1 ) {
				mikmodule_timedMessage(
					"MikMod Load Error",
					"Perhaps turn on 'subdirs' setting?" );
			}
		}
		return( NULL );
	}
	/* setup mikmod, load a single-song playlist */
	mikmodule_startup();
	mikGlob.playlist = plst;
	mikGlob.nTracks = playlist_count( plst );

	mikGlob.window = pz_new_window( "MikModule", PZ_WINDOW_NORMAL );
	if( pz_get_int_setting( mikGlob.config, CFG_FULLSCREEN ))
	{
                ttk_window_hide_header( mikGlob.window );
                mikGlob.window->x = 0;
                mikGlob.window->y = 0;
                mikGlob.window->w = ttk_screen->w;
                mikGlob.window->h = ttk_screen->h;
                mikGlob.w = ttk_screen->w;
                mikGlob.h = ttk_screen->h;
        } else {
		mikGlob.w = mikGlob.window->w;
		mikGlob.h = mikGlob.window->h;
	}

	mikGlob.widget = pz_add_widget( mikGlob.window, mikmodule_draw, mikmodule_event );
        pz_widget_set_timer( mikGlob.widget, 10 ); /* timer rate in ms */ /* __TR__ */

	/* load and start the song */
	mikmodule_trackAdvance( 1 );
	mikmodule_startTrack();

	/* make sure we don't go to sleep, etc */
	pz_set_priority( PZ_PRIORITY_VITAL );

        return( pz_finish_window( mikGlob.window ));
}


/* handle a menu browser request to open a single song */
PzWindow * mikmodule_open_handler( ttk_menu_item * item )
{               
	mikGlob.fromFile = 1;
	return( mikmodule_new_window_with_playlist(
			playlist_new_item( item->data )));
}


/* handle a menu browser request to open a playlist or directory */
PzWindow * mikmodule_open_playlist_handler( ttk_menu_item * item )
{
	int subdirs = pz_get_int_setting( mikGlob.config, CFG_RECURSE );

	if( !item || !item->data ) return( NULL );

	if( mikmodule_is_dir( item->data )) {
		mikGlob.fromFile = 0;
		if( subdirs == 2 ) {
			subdirs = pz_dialog( "Subdirectories?",
					"Include subdirectories also?",
					2, 5, "No", "Yes" );
		}

		return( mikmodule_new_window_with_playlist(
			playlist_directory( item->data, subdirs )));
	}

	mikGlob.fromFile = 1;
	return( mikmodule_new_window_with_playlist( 
			playlist_load( item->data )));
}


/* handle a menu browser request to open a playlist or directory, SHUFFLED */
PzWindow * mikmodule_open_playlist_shuffled_handler( ttk_menu_item * item )
{
	int subdirs = pz_get_int_setting( mikGlob.config, CFG_RECURSE );

	if( !item || !item->data ) return( NULL );

	if( mikmodule_is_dir( item->data )) {
		mikGlob.fromFile = 0;
		if( subdirs == 2 ) {
			subdirs = pz_dialog( "Subdirectories?",
					"Include subdirectories also?",
					2, 5, "No", "Yes" );
		}
		return( mikmodule_new_window_with_playlist(
			playlist_shuffle( playlist_directory( item->data, subdirs ))));
	} 

	mikGlob.fromFile = 1;
	return( mikmodule_new_window_with_playlist( 
			playlist_shuffle( playlist_load( item->data ))));
}


/* module initialization function */
void mikmodule_module_init() 
{
	/* internal module name */
	mikGlob.module = pz_register_module( "mikmodule", mikmodule_destroy );
	mikGlob.playlist = NULL;

	/* settings */
	mikGlob.config = pz_load_config( pz_module_get_cfgpath( mikGlob.module, "mikmodule.conf" ));
	if( pz_get_int_setting( mikGlob.config, CFG_READ ) != 1) {
		/* first-time init stuff */
		pz_set_int_setting( mikGlob.config, CFG_READ, 1 );
		pz_set_int_setting( mikGlob.config, CFG_VOLUME, 100 );
	}

	pz_menu_add_stub_group( "/Settings/MikModule", "Music" );

	pz_menu_add_setting_group( "/Settings/MikModule/Fadeout", 
				"Playback",
				CFG_FADEOUT,
				mikGlob.config, noyes_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Loops", 
				"Playback",
				CFG_LOOPS,
				mikGlob.config, noyes_options );

	pz_menu_add_setting_group( "/Settings/MikModule/Panning", 
				"Playback",
				CFG_PANNING,
				mikGlob.config, yesno_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Protracker", 
				"Playback",
				CFG_PROTRACKER,
				mikGlob.config, yesno_options );

	pz_menu_add_setting_group( "/Settings/MikModule/Separation", 
				"Playback",
				CFG_SEPARATION,
				mikGlob.config, highlow_options );

/*
	pz_menu_add_setting_group( "/Settings/MikModule/No. Buffers", 
				"Playlist",
				CFG_PLST_BUFFERS,
				mikGlob.config, buffer_options );
*/
	pz_menu_add_setting_group( "/Settings/MikModule/Subdirs", 
				"Behaviour",
				CFG_RECURSE,
				mikGlob.config, recurse_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Auto Dismiss Errors", 
				"Behaviour",
				CFG_AUTODISMISS,
				mikGlob.config, yesno_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Bleu's Heuristics", 
				"Behaviour",
				CFG_HEURISTICS,
				mikGlob.config, yesno_options );

	pz_menu_add_setting_group( "/Settings/MikModule/Full Screen", 
				"Display",
				CFG_FULLSCREEN,
				mikGlob.config, noyes_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Scootching", 
				"Display",
				CFG_SCOOTCHING,
				mikGlob.config, yesno_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Voice Dot Colors", 
				"Display",
				CFG_COLORDOTS,
				mikGlob.config, yesno_options );
	pz_menu_add_setting_group( "/Settings/MikModule/Color Gain", 
				"Display",
				CFG_COLORGAIN,
				mikGlob.config, gain_options );

	/* and sort it all... */
	pz_menu_sort( "/Settings/MikModule" );

	/* and create the browser single-song hook */
	mikGlob.fbx.name = N_( "Open with MikModule" );
	mikGlob.fbx.makesub = mikmodule_open_handler;
	pz_browser_add_action( mikmodule_ext_openable, &mikGlob.fbx );

	/* and create the browser playlist hook */
	mikGlob.fbpls.name = N_( "Open with MikModule" );
	mikGlob.fbpls.makesub = mikmodule_open_playlist_handler;
	pz_browser_add_action( mikmodule_ext_playlistable, &mikGlob.fbpls );

	/* and create the browser playlist shuffled hook */
	mikGlob.fbplss.name = N_( "Shuffle with MikModule" );
	mikGlob.fbplss.makesub = mikmodule_open_playlist_shuffled_handler;
	pz_browser_add_action( mikmodule_ext_playlistable, &mikGlob.fbplss );

	/* mikmod */
	MikMod_RegisterAllDrivers(); /* register all the drivers */
	MikMod_RegisterAllLoaders(); /* register all the module loaders */

	/* look for existing playlists to put them in our menu */
	mikmodule_scan_playlists();
	mikmodule_add_playlists_to_menu();
}


PZ_MOD_INIT (mikmodule_module_init)
