/* MikModule.h
 *
 *      interface to MikMod for podzilla 2
 *      
 *      (c)2007 Scott Lawrence
 *	   yorgle@gmail.com
 */

/*
 * Copyright (C) 2007 Scott Lawrence
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#define CFG_FADEOUT 	(0)	/* 0 = no,  1 = yes */
#define CFG_LOOPS 	(1)	/* 0 = no,  1 = yes */
#define CFG_PANNING 	(2)	/* 0 = yes, 0 = no */
#define CFG_PROTRACKER 	(3)	/* 0 = yes, 0 = no */
#define CFG_SEPARATION	(4)	/* 0 = none, 1 = mid, 2 = high */
#define CFG_LENSCAN	(5)	/* 0 = yes, 1 = no */

#define CFG_PLST_BUFFERS	(7) 	/* nbuffers = 1 << value */

#define CFG_VOLUME	(8)	/* current volume */
#define CFG_READ	(9)	/* have we read settings before? */
#define CFG_VIEWMODE	(10)	/* display state */
#define CFG_MESSAGES 	(11)	/* using PZ/VERBOSITY instead */
#define CFG_RECURSE	(12)	/* 0 = never, 1 = always, 2 = ask */
#define CFG_FULLSCREEN	(13)	/* 0 = no, 1 = yes */
#define CFG_SCOOTCHING	(14)	/* 0 = yes, 1 = no */

#define CFG_AUTODISMISS (15)	/* 0 = yes, 1 = no  - dismiss errors */
#define CFG_COLORDOTS	(16)	/* 0 = yes, 1 = no */
#define CFG_COLORGAIN   (17)	/* 0 = 1x, 1 = 2x, 2 = 4x */

#define CFG_HEURISTICS	(18)	/* 0 = yes, 1 = no */


/* for whichHeader */
#define MTW_FILENAME (0)
#define MTW_SONGNAME (1)
#define MTW_SONGPOS  (2)
#define MTW_MAX      (2)

/* the default period for headerTimeout */
#define MTW_TIMEOUT_TICKS	(75)


/* The MikModule render API structure.  
	This is generated just prior to render() being called, so it can
	be expected to be filled in and valid
*/

typedef struct MIK_RENDER {
	int minutes;
	int seconds;

	UBYTE used[UF_MAXCHAN];	/* 0 if voice is off, 1 if in use */
	int  inst[UF_MAXCHAN];	/* instrument playing in channel */
	long levl[UF_MAXCHAN];	/* current levels per channel */
	long pans[UF_MAXCHAN];	/* current pans per channel */
	long freq[UF_MAXCHAN];	/* current frequency per channel */

	long sampLen[UF_MAXCHAN];	/* sample length */
	long sampPos[UF_MAXCHAN];	/* sample pos */
	long sampSta[UF_MAXCHAN];	/* sample loop start */
	long sampEnd[UF_MAXCHAN];	/* sample loop end */
	char *sampNam[UF_MAXCHAN];	/* sample name */

	int nvoices;		/* number of voices */
	int nchans;		/* number of channels */

	int numpos;		/* number of patterns */
	int sngpos;		/* pattern being played */
	int maxpos;		/* length of the pattern */
	int patpos;		/* position in the pattern */

	/* stuff for the peaks */
	long peak[UF_MAXCHAN];	/* peak values */
	long peakPerTick;	/* dropoff of peaklevels per tick */
	long peakNextTick;	/* next tick time */

	long lmax;		/* maximum level */
	long lavg;		/* average level */
	long lmaxPeak;		/* max, peak dropoff */

	int hv;			/* height per voice */
	int ypad;		/* horizontal padding (remainder/2) */
	int wv;			/* width per voice */
	int xpad;		/* horizontal padding (remainder/2) */
} MIK_RENDER;


/* this is for the playlist scanner */
typedef struct playlist_launchable {
	char * shortname;
	char * fullpath;
	struct playlist_launchable * next;
} PLAYLIST_LAUNCHABLE;

/* and the primary global structure */
typedef struct myGlobals {
        PzModule * module;	/* this. */
        PzWindow * window;	/* our window */
        PzWidget * widget;	/* our display widget */
        PzConfig * config;	/* the config/settings */
	int w, h;		/* current display width and heighth */

	int paused;		/* are we paused? 1=yes */
	long ticks;		/* timer ticks since epoch */
	long updateTicks;	/* when to next poll the mikmod engine */
	int startedup;		/* is everything started up? */
	int consecutiveErrors;	/* number of consecutive errors */

	int scrubButton;	/* user is press-holding >> or << to scrub */
	int scrubTrigger;	/* used for telling if scrubbing started */

	int volume;		/* system volume */
	int editmode;		/* IDLE/Volume/scrub/... see below */
	int viewmode;		/* view mode number... see render.[ch] */

	VOICEINFO qv[UF_MAXCHAN];	/* queried voice info for rendering */
	int nqv;		/* number of voices queried */
				/* UF_MAXCHAN is defined in mikmod.h */
				/* this keeps this datablock off the stack */

	long volscrubEnd;	/* when will the vol/scrub overlay disappear? */

	char overlay[128];	/* top left overlay text */
	long overlayEnd;	/* top left overlay disappear time */

	long headerTimeout;	/* is it time to refresh the titlebar? */
	int whichHeader;	/* which header display */

	/* default display colors */
	ttk_color fg;		/* "window.FG" */
	ttk_color bg;		/* "window.BG" */
	ttk_color bg2;		/* "window.BG" mixed with .fg slightly */
	ttk_color rd;		/* "window.boRDer */

	/* colors for advanced rendering */
	ttk_color cols[32];	/* for 32 channels of colors */
	ttk_color colsHB[32];	/* for 32 channels of colors, dimmed */

	/* for our file browser hooks */
	ttk_menu_item fbx;	/* file execute */
	ttk_menu_item fbpls;	/* playlist execute */
	ttk_menu_item fbplss;	/* playlist shuffled */

	/* playlist related items */
	int nTracks;			/* number of items in the playlist */
	int fromFile;			/* is it a file based playlist? */
	PLAYLIST_ITEM * playlist; 	/* the entire playlist */
	PLAYLIST_ITEM * nowPlaying;	/* one item in that playlist */

	/* playlist lists */
	PLAYLIST_LAUNCHABLE * miklist;	/* list of launchable playlists */

	/* Render structure */
	MIK_RENDER mr;		/* the current values for render() */
} MYGLOBALS;


extern MYGLOBALS mikGlob;

#define EDIT_IDLE	(0)
#define ERIC_IDLE	(0)
#define TERRY_GILLIAM	(0)
#define TERRY_PRATCHET	(0)
#define RATCHET_WRENCH	(0)
#define BEER_WENCH	(0)
#define BEER_KEG	(0)
#define MEG_GRIFFIN	(0)
#define MORNING_COFFEE	(0)
#define MORNING_CROISSANT (0)
#define MORNINGTON_CRESCENT (0xff)
#define EDIT_VOLUME	(1)
#define EDIT_SCRUB	(2)
#define EDIT_VIEW	(3)
#define EDIT_FIRST	(0)
#define EDIT_MAX	(3)

/* for rendering functions... */
typedef void (*render_mik)( ttk_surface srf, MYGLOBALS * mglob, 
				int param, int flags ); 

typedef struct view_fcn {
        render_mik drawfcn;     /* the pointer to the draw function */
        int param;              /* parameter to be passed in */
} VIEW_FCN;

typedef struct view {
        char * name;            /* the name of the render view */
        VIEW_FCN fcns[6];
} VIEW;

#define VIEW_F_END	{ 0, 0 }
#define VIEW_F_SINGLE	( f, p )	{ f, p }, {0, 0}

#define VIEW_FLAG_NONE  (0)
#define VIEW_FLAG_FIRST (1)




void mikmodule_newOverlay( char * buf, int ticks );
void mikmodule_clearMikRender( MYGLOBALS * mglob );

void mikmodule_startup( void );
void mikmodule_shutdown( void );

int mikmodule_trackAdvance( int direction );
void mikmodule_startTrack( void );

char *zot_space( char *str );

