/* render
 *
 *      making purty drarings.
 *      
 *      (c)2007 Scott Lawrence
 *	   yorgle@gmail.com
 */

/*
 * Copyright (C) 2007 Scott Lawrence
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <string.h>
#include "mikmod.h"
#include "pz.h"
#include "playlist.h"
#include "mikmodule.h"
#include "version.h"

#define PEAK_DROPOFF (2000)
#define USE_RECTANGLES (1)


#define DRAW_SOFTRECT( s, x, y, a, b, c ) \
	do {\
		ttk_line( (s),  (x),   (y)+1,  (x),   (b)-1,  (c) );\
		ttk_line( (s),  (x)+1, (b),    (a)-1, (b),    (c) );\
		ttk_line( (s),  (a),   (b)-1,  (a),   (y)+1,  (c) );\
		ttk_line( (s),  (a)-1, (y),    (x)+1, (y),    (c) );\
	} while( 0 )
	

#ifdef USE_RECTANGLES
// rectangles - snappy!
#define DRAW_SHAPE( s, x, y, rx, ry, c ) \
	DRAW_SOFTRECT( (s), (x)-(rx)-1, (y)-(rx)-1, (x)+(rx), (y)+(ry), (c))

//	ttk_rect( (s), (x)-(rx), (y)-(ry), (x)+(rx), (y)+(ry), (c))

#define DRAW_FILLSHAPE( s, x, y, rx, ry, c ) \
	ttk_fillrect( (s), (x)-(rx), (y)-(ry), (x)+(rx), (y)+(ry), (c))

#else

// ellipses - SLOOOOOOW
#define DRAW_SHAPE( s, x, y, rx, ry, c ) \
	ttk_ellipse( (s), (x), (y), (rx), (ry), (c) )

#define DRAW_FILLSHAPE( s, x, y, rx, ry, c ) \
	ttk_fillellipse( (s), (x), (y), (rx), (ry), (c) )
#endif



void mikmodule_dottedVline( ttk_surface srf, int x, int y1, int y2, ttk_color c )
{
	int yy;
	for( yy = y1 ; yy < y2 ; yy+=2 )
		ttk_pixel( srf, x, yy, c );
}


void mikmodule_render_overlay( ttk_surface srf )
{
	int textw;
	int texth = ttk_text_height( ttk_menufont );

        if( mikGlob.ticks < mikGlob.overlayEnd && mikGlob.overlay[0] != '\0' ) {
		textw = ttk_text_width( ttk_menufont, mikGlob.overlay );

                ttk_fillrect( srf, 0, 0, textw+6, texth+6, mikGlob.bg );
                ttk_rect( srf, -1, -1, textw+7, texth+6, mikGlob.fg );
                ttk_rect( srf, -1, -1, textw+6, texth+7, mikGlob.fg );
		ttk_text( srf, ttk_menufont, 2, 2, mikGlob.fg, mikGlob.overlay );
        }
}


#define mikmodule_scaledYPos( mglob, val, max ) \
	( (val) * (mglob)->h / (max) )

#define mikmodule_scaledXPos( mglob, val, max ) \
	( (val) * (mglob)->w / (max) )


static void mikmodule_scaledHDot( ttk_surface srf, MYGLOBALS * mglob,
			      int yc, 
			      long hpos, long hmax,
			      long size, long smax,
			      ttk_color fill, ttk_color container )
{
	int xp = mikmodule_scaledXPos( mglob, hpos, hmax );
	int sz = size * (mglob->h>>2) / smax;
	DRAW_FILLSHAPE( srf, xp, yc, sz, sz, fill );
	DRAW_SHAPE( srf, xp, yc, sz, sz, container );
}

static void mikmodule_scaledVDot( ttk_surface srf, MYGLOBALS * mglob,
			      int xc, 
			      long hpos, long hmax,
			      long size, long smax,
			      ttk_color fill, ttk_color container )
{
	int yp = mikmodule_scaledYPos( mglob, hpos, hmax );
	int sz = size * (mglob->h>>2) / smax;
	DRAW_FILLSHAPE( srf, xc, mglob->h-yp, sz, sz, fill );
	DRAW_SHAPE( srf, xc, mglob->h-yp, sz, sz, container );
}

static void mikmodule_fixedHDot( ttk_surface srf, MYGLOBALS * mglob,
			      int yc, 
			      long hpos, long hmax,
			      int sz,
			      ttk_color fill, ttk_color container )
{
	int xp = mikmodule_scaledXPos( mglob, hpos, hmax );
	DRAW_FILLSHAPE( srf, xp, yc, sz, sz, fill );
	DRAW_SHAPE( srf, xp, yc, sz, sz, container );
}



/* currently, this is only used for the bottom third. can be cleaned up/replaced */
static void horizontalBox( ttk_surface srf, char * key,
				int y, int h, int max, int pos,
				int linefrom )
{
	char buf[80];
	int in = 2; /* inset from both sides */
	int gw;	/* mikGlobal width of the widget */
	int ow; /* one-segment width */
	int xp; /* x position for the slider */
	int xl; /* x position for the linefrom */
	ttk_color c; 

	if( linefrom == -2 ) {
		in = 2+h;
	}

	gw = mikGlob.w-(in*2);
	ow = gw/(max+1);

	if( pos < 0 ) pos = 0;
	if( pos > max ) pos = max-1;
	xp = pos * gw / (max+1);

	if( ow < 2 ) ow = 3;

	if( linefrom >= 0 )
	{
		if( linefrom < 0 ) linefrom = 0;
		if( linefrom >= max ) linefrom = max-1;
		xl = linefrom * gw/max;

		ttk_ap_fillrect( srf, ttk_ap_getx( buf ), 
				in+xp, y, in+xl, y+h );
	} else if( linefrom == -1 ) {
		snprintf( buf, 80, "%s.border", key );
		ttk_ap_rect( srf, ttk_ap_getx( buf ), in, y, in+gw, y+h );
	} else if( linefrom == -2 ) {
		snprintf( buf, 80, "%s.border", key );
		ttk_ap_rect( srf, ttk_ap_getx( buf ), in, y+(h/2), in+gw, y+(h/2) );
	}


	if( linefrom == -2 || linefrom == -3 ) {
                snprintf( buf, 80, "%s.fg", key );
                c = ttk_ap_getx( buf )->color;

                DRAW_FILLSHAPE( srf, in+xp, y+(h/2), (h/2), (h/2), c );
                snprintf( buf, 80, "%s.border", key );
                c = ttk_ap_getx( buf )->color;
                DRAW_SHAPE( srf, in+xp, y+(h/2), (h/2), (h/2), c );
	} else {
		snprintf( buf, 80, "%s.fg", key );
		ttk_ap_fillrect( srf, ttk_ap_getx( buf ), in+xp, y, in+xp+ow, y+h );
		snprintf( buf, 80, "%s.border", key );
		ttk_ap_rect( srf, ttk_ap_getx( buf ), in+xp, y, in+xp+ow, y+h );
	}
}


static void render_centerBarText( ttk_surface srf, char * text )
{
	int textw = ttk_text_width( ttk_menufont, text );
	int texth = ttk_text_height( ttk_menufont );
	ttk_color fg;

	int y = (mikGlob.h - texth)>>1;

	if( ttk_ap_getx( "menu.fg" ) ){
		fg = ttk_ap_getx( "menu.fg" )->color;
	} else {
		fg = ttk_makecol( 255, 255, 0 );
	}

	ttk_ap_fillrect( srf, ttk_ap_getx( "menu.bg" ), 0, y-texth, mikGlob.w, y+texth);
	ttk_rect( srf, 0, y-texth, mikGlob.w, y+texth, fg );
	ttk_rect( srf, 1, y-texth-1, mikGlob.w-1, y+texth+1, fg );
	ttk_text( srf, ttk_menufont, (mikGlob.w-textw)>>1, y-(texth/2), fg, text );
}


ttk_color mikmodule_paramColor( MYGLOBALS * mglob, int param, int v )
{
	int r,g,b;
	r=g=b = 0;

	if( param & 0x08 ) v = v>>1;
	if( param & 0x04 ) r=v;
	if( param & 0x02 ) g=v;
	if( param & 0x01 ) b=v;

	return( ttk_makecol( r, g, b ));
}

/************************************************************************/


#ifdef NOTHING
static void nothing( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
}
#endif


static void credittext( ttk_surface srf, MYGLOBALS * mglob, int y, char * txt )
{
	int textw = ttk_text_width( ttk_menufont, txt );
	ttk_text( srf, ttk_menufont, (mglob->w - textw)>>1, y, mglob->fg, txt );
}

static void mikmodule_view_credits( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	char buf[80];
	int v = 4; /* vertical text spacing */
	int texth = ttk_text_height( ttk_menufont );
        long engineversion = MikMod_GetVersion();

	snprintf( buf, 80, "MikModule v%s (%ld.%ld.%ld)",
			MIKMODULE_VERSION,
			(engineversion >> 16) & 255,
			(engineversion >> 8) & 255,
			(engineversion) & 255 );

	/* keep it small to fit in the mini's screen */
	credittext( srf, mglob, 2+(texth+v)*0, buf );
	credittext( srf, mglob, 2+(texth+v)*1, "BleuLlama@ipodlinux.org" );

	credittext( srf, mglob, 8+(texth+v)*2, "Thanks to libmikmod authors;" );
	credittext( srf, mglob, 8+(texth+v)*3, "Jean-Paul Mikkers, Jake Stine" );
	credittext( srf, mglob, 8+(texth+v)*4, "Miod Vallat, Raphael Assenat" );
	credittext( srf, mglob, 8+(texth+v)*5, "and all the rest..." );
}


static void mikmodule_view_information( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{ 
	int v = 2; /* vertical text spacing */
	int textw, texth;
	char buf[80];

	if( !mglob || !mglob->nowPlaying || !mglob->nowPlaying->song ) return;

	texth = ttk_text_height( ttk_menufont );

	ttk_text( srf, ttk_menufont, 2, 2+(texth+v)*0, mglob->fg, mglob->nowPlaying->shortname );
	ttk_text( srf, ttk_menufont, 2, 2+(texth+v)*1, mglob->fg,
		(mglob->nowPlaying->song->modtype)?mglob->nowPlaying->song->modtype:"?" );

	ttk_text( srf, ttk_menufont, 2, 2+(texth+v)*2, mglob->fg, mglob->nowPlaying->song->songname );
	v--;
	snprintf( buf, 80, "tim: %01d:%02d", mglob->mr.minutes, mglob->mr.seconds);
	ttk_text( srf, ttk_menufont, 2, 4+(texth+v)*3, mglob->fg, buf );

	snprintf( buf, 80, "pat: %03d/%03d  %02X",
			mglob->mr.sngpos,
			mglob->mr.numpos,
			mglob->mr.patpos );
	ttk_text( srf, ttk_menufont, 2, 4+(texth+v)*4, mglob->fg, buf );

	snprintf( buf, 80, "spd: %d/%d",
			mglob->nowPlaying->song->bpm,
			mglob->nowPlaying->song->sngspd);
	ttk_text( srf, ttk_menufont, 2, 4+(texth+v)*5, mglob->fg, buf );

	snprintf( buf, 80, "chn: %d/%d",
			mglob->nowPlaying->song->realchn, 
			mglob->nowPlaying->song->numchn );
	ttk_text( srf, ttk_menufont, 2, 4+(texth+v)*6, mglob->fg, buf );


	/* playlist info */
	if( mglob->nTracks > 1 ) {
		/* only show playlist count if we have a playlist of >1 songs */
		snprintf( buf, 80, "pls: %d/%d", mglob->nowPlaying->index, mglob->nTracks );
		textw = ttk_text_width( ttk_menufont, buf );
		ttk_text( srf, ttk_menufont, 2, 4+(texth+v)*7, mglob->fg, buf );
	}
}


static void mikmodule_view_songposdots( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int y = mglob->h * mglob->mr.sngpos / mglob->mr.numpos;
	ttk_fillrect( srf, mglob->w-5, y, mglob->w, y+5, mglob->rd );
	ttk_rect( srf, mglob->w-5, y, mglob->w, y+5, mglob->fg );

	y = mglob->h * mglob->mr.patpos / mglob->mr.maxpos;
	ttk_fillrect( srf, mglob->w-4, y, mglob->w, y+3, mglob->rd );
	ttk_rect( srf, mglob->w-4, y, mglob->w, y+3, mglob->fg );
}


static void mikmodule_view_voices( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i;
	int sx, ex;
	int ty, by, cy;
	int dd;
	int texth = ttk_text_height( ttk_menufont );
	char buf[80];

	for( i=0 ; i<mglob->mr.nvoices ; i++ )
	{
		ty = mglob->mr.ypad + (mglob->mr.hv * i);
		by = mglob->mr.ypad + (mglob->mr.hv * (i+1));
		cy = (ty + by) >> 1;

		/* content */
		if( mglob->mr.used[i] ) {

			/* sample loop */
			sx = mglob->mr.sampSta[i];
			ex = mglob->mr.sampEnd[i];
			if( ex > 2 ) {
				sx = mikmodule_scaledXPos( mglob, sx, mglob->mr.sampLen[i] );
				ex = mikmodule_scaledXPos( mglob, ex, mglob->mr.sampLen[i] );
				if( ex>mglob->w-1) ex = mglob->w-1;
				/* left vert */
				ttk_line( srf, sx, cy-1, sx, cy+1, mglob->rd );
				/* horiz */
				ttk_line( srf, sx, cy,  ex, cy, mglob->rd );
				/* right vert */
				ttk_line( srf, ex, cy-1, ex, cy+1, mglob->rd );
			}

			/* position indicator */
			sx = mikmodule_scaledXPos( mglob, mglob->mr.sampPos[i], mglob->mr.sampLen[i] );
			/* bottom line */
			ttk_line( srf, 0, by-1, sx, by-1, mglob->fg );
			/* vert */
			ttk_line( srf, sx, by-1, sx, cy, mglob->fg );

			/* volume dot */
			if( mglob->mr.nvoices < 8 ) {
				dd = mglob->mr.levl[i]>>9;
			} else {
				dd = (mglob->h>>1) * mglob->mr.levl[i] / 65536;
			}
			dd = mglob->mr.levl[i] * (mglob->mr.hv/2) / 65535;
			DRAW_FILLSHAPE( srf, sx, cy, dd, dd, mglob->cols[ mglob->mr.inst[i]&0x1f] );
			DRAW_SHAPE( srf, sx, cy, dd, dd, mglob->colsHB[ mglob->mr.inst[i]&0x1f] );

			/* text */
			if( (param & 0x01) && (mglob->mr.hv > texth) ) {
				if( mglob->mr.sampNam[i] != NULL ) {
					snprintf( buf, 80, "%02d  %s", mglob->mr.inst[i],
						mglob->mr.sampNam[i] );
				} else {
					snprintf( buf, 80, "%02d", mglob->mr.inst[i] );
				}
				ttk_text( srf, ttk_menufont, 1, ty, mglob->colsHB[ mglob->mr.inst[i] & 0x01f], buf );
				ttk_text( srf, ttk_menufont, 2, ty, mglob->colsHB[ mglob->mr.inst[i] & 0x01f], buf );
				ttk_text( srf, ttk_menufont, 1, ty-1, mglob->cols[ mglob->mr.inst[i] & 0x01f], buf );
			}
		}
	}
}



static void mikmodule_view_volumebars( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i, yp;
	TApItem * ta = ttk_ap_getx( "slider.full" );

	/* display volume thingies */
	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.used[i] ) {
			yp = mglob->h - mikmodule_scaledYPos( mglob, mglob->mr.levl[i], 65535 );
			ttk_ap_fillrect( srf, ta, 
				mglob->mr.xpad + (i * mglob->mr.wv), yp,
				mglob->mr.xpad + ((i+1) * mglob->mr.wv),
				mglob->h );
		}
	} 
}


static void mikmodule_view_kittbars( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i, yp;
	int m;
	ttk_color c;

	/* this is totally wrong - it should be large bars when volume is high,
	   but it appears to be the opposite.  I know what's wrong and how to 
	   fix it, but i really like the way it looks, so I'm leaving it be. */

	/* i love graphics programming... hehehe */

	m = MIN( 255, mglob->mr.lmax>>7);
	c = ttk_makecol( m, 0, 0 );

	/* display 'em */
	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.used[i] ) {

			yp = mikmodule_scaledYPos( mglob, mglob->mr.levl[i], 65535 );
			ttk_fillrect( srf, 
				mglob->mr.xpad + (i * mglob->mr.wv), 
				mglob->h-yp,
				mglob->mr.xpad + ((i+1) * mglob->mr.wv),
				yp, c );
		}
	} 
}

static void mikmodule_view_brightbars( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i;
	long vv;
	ttk_color c;
	int r, g, b;

	/* display volume thingies */
	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.used[ i ] ) {
			if( mglob->mr.levl[i] > 0 ) {
				vv = mglob->mr.levl[i];
				if( param == 0 ) {
					ttk_unmakecol( 
						mglob->cols[mglob->mr.inst[i]&0x1f],
						&r, &g, &b );

					vv <<= pz_get_int_setting( mikGlob.config, CFG_COLORGAIN );
				
					r = r * (vv>>8) / 256;
					g = g * (vv>>8) / 256;
					b = b * (vv>>8) / 256;
				} else if( param == 1 ) {
					r = g = b = vv>>8;
				} else {
					g = b = 0;
					r = vv>>8;
				}

				/* drop it down to 240 due to puple issues */
				if( r > 240 ) r = 240;
				if( g > 255 ) g = 255;
				if( b > 255 ) b = 255;

				c = ttk_makecol( r, g, b );

				ttk_fillrect( srf, 
					mglob->mr.xpad + (i*mglob->mr.wv), 0,
					mglob->mr.xpad + ((i+1)*mglob->mr.wv),
					mglob->h, c );
			}
		}
	} 
}


static void mikmodule_tuxdot( ttk_surface srf, int x, int y, int param, ttk_color c1, ttk_color c2 )
{
	if( param == 0 ) {
		ttk_draw_icon_color( ttk_icon_tux, srf, x-16, y-16, c1, 0 );
		return;
	}

	if( param == 1 ) {
		DRAW_FILLSHAPE( srf, x, y, 18, 18, c1 );
		DRAW_SHAPE( srf, x, y, 18, 18, c2 );
	}
	ttk_draw_icon_color( ttk_icon_tux, srf, x-16, y-16, 0, c1 );
}

static void mikmodule_view_dancingtux( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i, yp;
	TApItem * ta = ttk_ap_getx( "slider.full" );

	/* display volume thingies */
	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.used[i] ) {

			yp = mglob->h - mikmodule_scaledYPos( mglob, mglob->mr.peak[i], 65535 );
			mikmodule_tuxdot( srf, mglob->mr.xpad + (i * mglob->mr.wv) + (mglob->mr.wv>>1),
					yp, param,
					mglob->cols[ mglob->mr.inst[i] ],
					mglob->colsHB[ mglob->mr.inst[i] ] );
		}
	} 
}


static void mikmodule_view_pandots( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i;

	mikmodule_dottedVline( srf, mglob->w>>1, 0, mglob->h, mglob->bg2 );
	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.used[i] ) {

			mikmodule_fixedHDot( srf, mglob, 
				(mglob->mr.hv * i)
					+ mglob->mr.ypad 
					+ (mglob->mr.hv>>1),
				mglob->mr.pans[i],
				PAN_RIGHT,
				(mglob->mr.hv>>1)-1,
				mglob->mr.levl[i]<16000?
				    mglob->colsHB[ mglob->mr.inst[i]&0x1f ] :
				    mglob->cols[ mglob->mr.inst[i]&0x1f ],
				mglob->mr.levl[i]>31768?
				0xffff:
				mglob->colsHB[ mglob->mr.inst[i]&0x1f ] );
		}
	}
}

static void mikmodule_view_freqdots( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i;
	long vv;

	if( param ) { 
		for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
			vv = mglob->mr.levl[i];
			if( vv > 0 ) {
				mikmodule_scaledVDot( srf, mglob, 
					(mglob->mr.wv * i)
						+ mglob->mr.xpad 
						+ (mglob->mr.wv>>1),
					mglob->mr.freq[i],
					mglob->nowPlaying->maxfreq,
					mglob->mr.levl[i],
					65535,
					mglob->cols[ mglob->mr.inst[i]&0x1f ],
					mglob->colsHB[ mglob->mr.inst[i]&0x1f ] );
			}
		}
	} else {
		for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
			vv = mglob->mr.levl[i];
			if( vv > 0 ) {
				mikmodule_scaledHDot( srf, mglob, 
					(mglob->mr.hv * i)
						+ mglob->mr.ypad 
						+ (mglob->mr.hv>>1),
					mglob->mr.freq[i],
					mglob->nowPlaying->maxfreq,
					mglob->mr.levl[i],
					65535,
					mglob->cols[ mglob->mr.inst[i]&0x1f ],
					mglob->colsHB[ mglob->mr.inst[i]&0x1f ] );
			}
		}
	}
}


static void mikmodule_view_volrainbow( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i;
	int bw;
	int iw, iww;
	int vv;
	int yy;
	int w = mglob->mr.wv;
	int xoff = mglob->mr.xpad;

	/* NOTE: one speedup is that we could always just render the 
		 background image (the gradients and such) first, and then
		 just overlay the black over it.
	*/
	ttk_color black = ttk_makecol( BLACK );
	ttk_color red   = ttk_makecol( 255, 0, 0 );
	ttk_color yel   = ttk_makecol( 255, 255, 0 );
	ttk_color grn   = ttk_makecol( 0, 100, 0 );

	ttk_color dkred = ttk_makecol( 128, 0, 0 );
	ttk_color dkyel = ttk_makecol( 128, 128, 0 );
	ttk_color dkgrn = ttk_makecol( 0, 50, 0 );

	ttk_color ltred = ttk_makecol( 255, 128, 128 );
	ttk_color ltyel = ttk_makecol( 255, 255, 128 );
	ttk_color ltgrn = ttk_makecol( 80, 100, 80 );
	
	/* backfill with the gradient */
	ttk_vgradient( srf, 0, 0, mglob->w, mglob->h*3/4, red, yel );
	ttk_vgradient( srf, 0, mglob->h*3/4, mglob->w, mglob->h, yel, grn );

	bw = (w<8)?1:3;	/* bar shade width */

	/* draw the bevels */
        for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		iw = i*w;
		iww = iw+w;

		ttk_vgradient( srf, xoff+iw, 0, xoff+iw+bw, mglob->h*3/4, dkred, dkyel );
		ttk_vgradient( srf, xoff+iw, mglob->h*3/4, xoff+iw+bw, mglob->h, dkyel, dkgrn );

		ttk_vgradient( srf, xoff+iww-bw, 0, xoff+iww, mglob->h*3/4, ltred, ltyel );
		ttk_vgradient( srf, xoff+iww-bw, mglob->h*3/4, xoff+iww, mglob->h, ltyel, ltgrn );
	}

	/* mask off the offset/overlap */
	if( xoff ) {
		ttk_fillrect( srf, 0, 0, xoff, mglob->h, black );
		ttk_fillrect( srf, mglob->w-xoff-1, 0, mglob->w, mglob->h, black );
	}

	/* draw the volume bars */
        for( i=0 ; i<mglob->mr.nvoices ; i++ ) {

		vv = mglob->mr.levl[i];
		yy = mglob->h - (mglob->h * vv / 65535);
		iw = i*w;
		iww = iw+w;
		ttk_fillrect( srf, xoff+iw, 0, xoff+iww, yy, black ); 

		/* borders */
/*
		if( w > 8 ) {
			ttk_fillrect( srf, xoff+iw, 0, xoff+iw+4, mglob->h, black );
			ttk_fillrect( srf, xoff+iww-4, 0, xoff+iww, mglob->h, black );
		}
*/
	}
}

static void mikmodule_view_peaks( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i, iw, yy;
	/* funky 
	int yy2;
	*/

	/* draw the peaks */
	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.peak[i] ) {
			iw = i*mglob->mr.wv;
			yy = mglob->h - (mglob->h * mglob->mr.peak[i] / 65535);
			ttk_rect( srf, mglob->mr.xpad+iw, yy-1,
					mglob->mr.xpad+iw+mglob->mr.wv, yy+1, 
					mikmodule_paramColor( mglob, param, 65535 ));
		}
	}

	/* funky 
	yy = mglob->h - (mglob->h * mglob->mr.lmax / 65535);
	yy2 = mglob->h - (mglob->h * mglob->mr.lmaxPeak / 65535);
	ttk_fillrect( srf, 0, yy, mglob->w, yy2, param>>1 );
	*/
}

static void mikmodule_view_theremin( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int xx, yy, dd, i;
	long vv, ff, pp;

	for( i=0 ; i<mglob->mr.nvoices ; i++ ) {
		if( mglob->mr.used[i] ) {
			vv = mglob->mr.levl[i];
			ff = mglob->mr.freq[i];
			pp = mglob->mr.pans[i];

			/* x is pan */
			xx = (pp * mglob->w / PAN_RIGHT)-1; /* -1 to move undefined ones off the left */

			/* y is frequency */
			yy = mglob->h - (ff * (mglob->h-10) / mglob->nowPlaying->maxfreq) +5;

			/* d is volume */
			dd = vv * (mglob->h/4) / 65535;

			/* color is instrument */
			if( !pz_get_int_setting( mikGlob.config, CFG_COLORDOTS ) ) {
			    DRAW_FILLSHAPE( srf, xx, yy, dd, dd, mglob->cols[mglob->mr.inst[i]&0x1f] );
			    DRAW_SHAPE( srf, xx, yy, dd, dd, mglob->colsHB[mglob->mr.inst[i]&0x1f] );
			} else {
			    DRAW_FILLSHAPE( srf, xx, yy, dd, dd, mglob->fg );
			    DRAW_SHAPE( srf, xx, yy, dd, dd, mglob->rd );
			}
		}
	}
}


static void mikmodule_view_colorshift( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int r, g, b;

	r = 255 * mglob->mr.sngpos / mglob->mr.numpos;
	g = 255 * mglob->mr.patpos / mglob->mr.maxpos;
	b = mglob->mr.lmax >> 8;
	ttk_fillrect( srf, 0, 0, mglob->w, mglob->h, ttk_makecol( r,g,b ) );
}

static void mikmodule_view_bg_black( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	ttk_fillrect( srf, 0, 0, mglob->w, mglob->h, ttk_makecol( BLACK ) );
}

static void mikmodule_view_bg_ap( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	ttk_ap_fillrect( srf, ttk_ap_getx( "slider.bg" ), 0, 0, mglob->w, mglob->h );
}

static void mikmodule_view_bg_alternating( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	int i, ty, by;
	for( i=0 ; i<mglob->mr.nvoices ; i++ )
	{
		ty = mglob->mr.ypad + (mglob->mr.hv * i);
		by = mglob->mr.ypad + (mglob->mr.hv * (i+1));

		/* background */
		ttk_fillrect( srf, 0, ty, mglob->w, by, (i&1)? mglob->bg : mglob->bg2 );
	}
}

#ifndef ABS
#define ABS( a )  (((a)<0)?-(a):(a))
#endif


static void mikmodule_view_bg_peak( ttk_surface srf, MYGLOBALS * mglob, int param, int flags )
{
	long v = ABS( (mglob->mr.lmaxPeak + mglob->mr.lmax)>>1 )>>8;

	ttk_fillrect( srf, 0, 0, mglob->w, mglob->h, 
			mikmodule_paramColor( mglob, param, v ));
}


/************************************************************************/

static VIEW mik_views[] = {
	{ "Information", {
		{ mikmodule_view_information, 0 },
		VIEW_F_END } },

	{ "Voices", { 
		{ mikmodule_view_bg_alternating, 0 },
		{ mikmodule_view_voices, 1 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },
	{ "Voices NT", {
		{ mikmodule_view_bg_alternating, 0 },
		{ mikmodule_view_voices, 0 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },
	{ "Voices + Freq", {
		{ mikmodule_view_bg_alternating, 0 },
		{ mikmodule_view_voices, 1 },
		{ mikmodule_view_freqdots, 0 },
		VIEW_F_END } },
	{ "Voices + Freq NT", {
		{ mikmodule_view_bg_alternating, 0 },
		{ mikmodule_view_voices, 0 },
		{ mikmodule_view_freqdots, 0 },
		VIEW_F_END } },

	{ "Volume Bars", {
		{ mikmodule_view_bg_ap, 0 },
		{ mikmodule_view_volumebars, 0 },
		{ mikmodule_view_peaks, 0x0 },
		VIEW_F_END } },
	{ "Vol Bars + Freq", {
		{ mikmodule_view_bg_ap, 0 },
		{ mikmodule_view_volumebars, 0 },
		{ mikmodule_view_freqdots, 1 },
		VIEW_F_END } },

	{ "Volume Rainbow", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_volrainbow, 0 },
		{ mikmodule_view_peaks, 0x06 },
		VIEW_F_END } },
	{ "Rainbow + Freq", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_volrainbow, 0 },
		{ mikmodule_view_peaks, 0x06 },
		{ mikmodule_view_freqdots, 1 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },

	{ "KITT Bars", {
		{ mikmodule_view_bg_peak, 0x08 | 0x04 },
		{ mikmodule_view_kittbars, 0 },
		VIEW_F_END } },

	{ "Bright Bars", {
		{ mikmodule_view_bg_peak, 0 },
		{ mikmodule_view_brightbars, 0 },
		VIEW_F_END } },
	{ "Bright Bars + Peaks", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_brightbars, 0 },
		{ mikmodule_view_peaks, 0x07 },
		VIEW_F_END } },
	{ "Bright Bars + Freq", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_brightbars, 0 },
		{ mikmodule_view_freqdots, 1 },
		VIEW_F_END } },
	{ "White Bars", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_brightbars, 1 },
		VIEW_F_END } },
	{ "Red Bars", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_brightbars, 2 },
		VIEW_F_END } },

	{ "Pan Dots", {
		{ mikmodule_view_bg_alternating, 0 },
		{ mikmodule_view_pandots, 0 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },

	{ "Frequency Dots (H)", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_freqdots, 0 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },

	{ "Frequency Dots (V)", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_freqdots, 1 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },

	{ "Theremin", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_theremin, 0 },
		VIEW_F_END } },

	{ "Vol Bars + Theremin", {
		{ mikmodule_view_bg_ap, 0 },
		{ mikmodule_view_volumebars, 0 },
		{ mikmodule_view_theremin, 0 },
		VIEW_F_END } },

	{ "Dancing Tux", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_dancingtux, 0 },
		VIEW_F_END } },
	{ "Dancing Tux II", {
		{ mikmodule_view_bg_black, 0 },
		{ mikmodule_view_dancingtux, 1 },
		VIEW_F_END } },

	{ "Colorshift", {
		{ mikmodule_view_colorshift, 0 },
		{ mikmodule_view_songposdots, 0 },
		VIEW_F_END } },

	{ "Colorshift + Tux", {
		{ mikmodule_view_colorshift, 0 },
		{ mikmodule_view_songposdots, 0 },
		{ mikmodule_view_dancingtux, 2 },
		VIEW_F_END } },

	{ "Credits", {
		{ mikmodule_view_credits, 0 },
		VIEW_F_END } },

	{ NULL, { { NULL, 0 } } }
};

int mikmodule_getNumViews( void )
{
	int p = 0;
	while( mik_views[ ++p ].name != NULL );
	return( p-1 );
}

void mikmodule_nextView( void )
{
	char buf[80];
	mikGlob.viewmode++;
	if( mikGlob.viewmode > mikmodule_getNumViews() ) 
		mikGlob.viewmode = 0;
	snprintf( buf, 80, "< %s >", mik_views[ mikGlob.viewmode ].name );
	mikmodule_newOverlay( buf, 30 );
}

void mikmodule_prevView( void )
{
	char buf[80];
	mikGlob.viewmode--;
	if( mikGlob.viewmode < 0 ) 
		mikGlob.viewmode = mikmodule_getNumViews();
	snprintf( buf, 80, "< %s >", mik_views[ mikGlob.viewmode ].name );
	mikmodule_newOverlay( buf, 30 );
}

/************************************************************************/

static void mikmodule_renderBottomThird( ttk_surface srf, MYGLOBALS * mglob )
{
	char buf[80];

	/* the "bottom third" overlay */
	if( mglob->ticks < mglob->volscrubEnd ) {
		switch( mglob->editmode ) {
		case( EDIT_VOLUME ):
			mikmodule_newOverlay( "", 0 );
			/* container */
			ttk_fillrect( srf, 0, mglob->h-22, mglob->w, mglob->h, mglob->bg );
			ttk_line( srf, 0, mglob->h-23, mglob->w, mglob->h-23, mglob->fg );
			ttk_line( srf, 0, mglob->h-24, mglob->w, mglob->h-24, mglob->fg );

			/* icons */
			ttk_draw_icon_color( pz_icon_spkr_off, srf, 2, 
					mglob->h-23+PZ_ICON_H(pz_icon_spkr_off)/2,
					mglob->fg, mglob->bg );
			ttk_draw_icon_color( pz_icon_spkr_loud, srf,
					mglob->w-2-PZ_ICON_W(pz_icon_spkr_loud),
					mglob->h-23+PZ_ICON_H(pz_icon_spkr_loud)/2,
					mglob->fg, mglob->bg );

			/* handle */
			horizontalBox( srf, "box.selected",
				mglob->h-20, 18, 128, mglob->volume, -2 );
			break;

		case( EDIT_SCRUB ):
			mikmodule_newOverlay( "", 0 );
			/* draw the background */
			ttk_fillrect( srf, 0, mglob->h-22, mglob->w, mglob->h, mglob->bg );
			ttk_line( srf, 0, mglob->h-23, mglob->w, mglob->h-23, mglob->fg );
			ttk_line( srf, 0, mglob->h-24, mglob->w, mglob->h-24, mglob->fg );

			/* icons */
/*
			ttk_draw_icon_color( pz_icon_play, srf, 5,
					mglob->h - PZ_ICON_H(pz_icon_play) - 4,
					mglob->fg, mglob->bg );
*/

			/* position */
			horizontalBox( srf, "box.selected",
				    	mglob->h-20, 10,
					mglob->nowPlaying->song->numpos,
				    	mglob->nowPlaying->song->sngpos, -1 );
			horizontalBox( srf, "box.default", 
					mglob->h-8, 6, 
					mglob->nowPlaying->maxpos, 
					mglob->nowPlaying->song->patpos, -1 );
			break;

		case( EDIT_VIEW ):
			snprintf( buf, 80, "< %s >", mik_views[ mikGlob.viewmode ].name );
			mikmodule_newOverlay( buf, 30 );
			break;
		}
	} else {
		mglob->editmode = EDIT_VOLUME;
	}
}


/************************************************************************/


void mikmodule_populateMikRender( MYGLOBALS * mglob )
{
	SAMPLE *splaying;
	VOICEINFO qv[UF_MAXCHAN];	/* queried voice info.  UF_MAXCHAN in mikmod.h */
	int nqv;
	int v, s;

	mglob->mr.peakPerTick = PEAK_DROPOFF;
	nqv = Player_QueryVoices( UF_MAXCHAN, qv );

	mglob->mr.lmax = mglob->mr.lavg = 0;

	for( v=0 ; v<nqv ; v++ )
	{
		/* general voice info */
		if( Voice_Stopped( v ) || !qv[v].s ) {
			mglob->mr.used[v] = 0;
			mglob->mr.levl[v] = 0;
			mglob->mr.sampNam[v] = NULL;
		} else {
			mglob->mr.used[v] = 1;

			/* the current instrument playing */
			for( s=0 ; s<mglob->nowPlaying->song->numsmp /*ins*/ ; s++ ) {
				splaying = &mglob->nowPlaying->song->samples[s];

				if( splaying == qv[v].s ) {
					mglob->mr.inst[v] = s;
				}
			}

			/* and the sample position, length, and loop */
			mglob->mr.pans[v] = Voice_GetPanning( v );
			mglob->mr.freq[v] = Voice_GetFrequency( v );
		        if( !pz_get_int_setting( mikGlob.config, CFG_SCOOTCHING ) ) {
				if( mglob->mr.freq[v] > mglob->nowPlaying->maxfreq ) {
					mglob->nowPlaying->maxfreq = mglob->mr.freq[v];
				}
			}

			if( qv[v].i && qv[v].i->insname )
				mglob->mr.sampNam[v] = qv[v].i->insname;
			else
				mglob->mr.sampNam[v] = qv[v].s->samplename;
			mglob->mr.sampLen[v] = qv[v].s->length;
			mglob->mr.sampPos[v] = Voice_GetPosition( v );
			mglob->mr.sampSta[v] = qv[v].s->loopstart;
			mglob->mr.sampEnd[v] = qv[v].s->loopend;

			/* current levels */
			mglob->mr.levl[v] = Voice_RealVolume( v )
					  * Voice_GetVolume( v ) / 256;

			mglob->mr.lavg += mglob->mr.levl[v];
			mglob->mr.lmax = MAX(mglob->mr.lmax, mglob->mr.levl[v]);
		}
	}
	if( mglob->mr.nvoices ) mglob->mr.lavg /= mglob->mr.nvoices;
	

	/* adjust peaks */
	if( mglob->ticks >= mglob->mr.peakNextTick ) {
		for( v=0 ; v<nqv ; v++ )
		{
			/* drop down */
			mglob->mr.peak[v] -= mglob->mr.peakPerTick;

			/* check for new peak */
			if( mglob->mr.levl[v] > mglob->mr.peak[v] ) {
				mglob->mr.peak[v] = mglob->mr.levl[v];
			}

			/* set up for next timer */
			mglob->mr.peakNextTick = mglob->ticks + 1;
		}
	}
	mglob->mr.lmaxPeak -= mglob->mr.peakPerTick;
	if( mglob->mr.lmax > mglob->mr.lmaxPeak )
	{
		mglob->mr.lmaxPeak = mglob->mr.lmax;
	}

	mglob->mr.nvoices = nqv;
	mglob->mr.nchans = mglob->nowPlaying->song->numchn;

	mglob->mr.numpos = mglob->nowPlaying->song->numpos;
	mglob->mr.sngpos = mglob->nowPlaying->song->sngpos;

	mglob->mr.maxpos = mglob->nowPlaying->maxpos;
	mglob->mr.patpos = mglob->nowPlaying->song->patpos;

	/* adjust for scootching */
	if( !pz_get_int_setting( mikGlob.config, CFG_SCOOTCHING ) ) {
		if( mglob->mr.patpos > mglob->nowPlaying->maxpos ) {
			mglob->mr.maxpos = 
			mglob->nowPlaying->maxpos = mglob->mr.patpos;
		}
	}

	mglob->mr.hv = mglob->h/mglob->mr.nchans;
	mglob->mr.ypad = (mglob->h - (mglob->mr.hv * mglob->mr.nchans))>>1;

	mglob->mr.wv = mglob->w/mglob->mr.nchans;
	mglob->mr.xpad = (mglob->w - (mglob->mr.wv * mglob->mr.nchans))>>1;
}


void mikmodule_clearMikRender( MYGLOBALS * mglob )
{
	memset( &mglob->mr, 0, sizeof( MIK_RENDER ));
}



void mikmodule_draw( PzWidget *widget, ttk_surface srf )
{
	int visx;

	/* if we're not valid... */
	if( !mikGlob.startedup || !Player_Active() ) {
		render_centerBarText( srf, N_( "Loading..." ));
		return;
	}

	if( !mikGlob.nowPlaying || !mikGlob.nowPlaying->song ) return;

	/* fill in the MIK_RENDER structure */
	mikmodule_populateMikRender( &mikGlob );

	/* draw the currently active visualizer(s) */
	if( mik_views[ mikGlob.viewmode ].fcns ) {
		for( visx=0 
		   ; mik_views[ mikGlob.viewmode ].fcns[visx].drawfcn != NULL 
		   ; visx++ ) {

		mik_views[ mikGlob.viewmode ].fcns[visx].drawfcn(
			srf, &mikGlob,
			mik_views[ mikGlob.viewmode ].fcns[visx].param, 
			(visx == 0)? VIEW_FLAG_FIRST : VIEW_FLAG_NONE );
		}
	}

	/* finally, draw the overlays and informational displays */
	mikmodule_renderBottomThird( srf, &mikGlob );
	if( Player_Paused() ) {
		render_centerBarText( srf, N_( "Paused" ));
	}
	mikmodule_render_overlay( srf );
}
