/* scan - playlist scanning
 *
 *      (c)2007 Scott Lawrence
 *         yorgle@gmail.com
 */

/* thanks to Courtc for his awesome debugging skills */

/*
 * Copyright (C) 2007 Scott Lawrence
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */


#include <sys/types.h>  /* readdir(), is_dir */
#include <dirent.h>     /* readdir() */
#include <unistd.h>
#include <string.h>
#include <ctype.h>	/* isspace */
#include "ttk.h"	/* TTK_EV_UNUSED */
#include "pz.h"
#include "mikmod.h"
#include "playlist.h"
#include "mikmodule.h"
#include "render.h"
#include "ext.h"
#include "scan.h"
#include "version.h"

/* all of the locations we'll look for playlists.
   this seems like a pretty extensive list, imo. */
static char *searchpaths[] = {
	"/",
	"/mods/",
	"/mods/playlists/",
	"/music/",
	"/music/playlists/",
	"/music/mods/",
	"/music/mods/playlists/",

	"/mnt/",
	"/mnt/mods/",
	"/mnt/mods/playlists",
	"/mnt/music/",
	"/mnt/music/playlists/",
	"/mnt/music/mods/",
	"/mnt/music/mods/playlists/",
	NULL
};


/* _launch
 *	just a menu hook/launcher to start the specific playlist file
 */
PzWindow *mikmodule_launch( ttk_menu_item * mi )
{
	if( pz_get_int_setting( pz_global_config, SHUFFLE ) == SHUFFLE_OFF )
		return( mikmodule_open_playlist_handler( mi ));
	return( mikmodule_open_playlist_shuffled_handler( mi ));
}


/* _add_playlists_to_menu
 *	scan through the PLAYLIST_LAUNCHABLE list, and add all nodes into
 *	the podzilla2 menu structure in /Music/Playlists/xxx (MikModule)
 */
void mikmodule_add_playlists_to_menu( void )
{
	int c=0;
	int len;
	char * ts = NULL;
	char menupath[64];
	PLAYLIST_LAUNCHABLE *pl = mikGlob.miklist;
	
	if( pl == NULL ) return;

	pz_menu_add_stub_group( "/Music", "Media" );
	while( pl ) {
		snprintf( menupath, 64, "/Music/Playlists/%s", pl->shortname );
		ttk_menu_item * mi;
		mi = pz_menu_add_action_group( menupath, "MikModule", mikmodule_launch );
		if( mi ) {

			// first, make a nice displayable version
			ts = strdup( pl->shortname );
			len = strlen(ts);
			// remove .ext
			for( c=len ; c>0 ; c-- ) {
				if( ts[c] == '.' ) {
					ts[c] = '\0';
					break;
				}
			}
			// change underscores to spaces
			len = strlen(ts);
			for( c=0 ; c<len ; c++ )
				if( ts[c] == '_' ) ts[c] = ' ';
			
			
			// now add it all to the menu node
			mi->name = ts;
			mi->free_name = 1;
			mi->data = strdup( pl->fullpath );
			mi->free_data = 1;
			mi->makesub = mikmodule_launch;
		}
		pl = pl->next;
	}
	pz_menu_sort( "/Music/Playlists" );
}


/* _scan_add_playlist
 *	add a playlist <dirpath>/<di>
 *	This creates the nodes in the PLAYLIST_LAUNCHABLE list in the global
 */
void mikmodule_scan_add_playlist( char * dirpath, struct dirent * di )
{
	char path[256];
	int len = strlen( di->d_name );
	PLAYLIST_LAUNCHABLE * pl;

	snprintf( path, 255-len, "%s", dirpath );
	if( path[ strlen(path)-1 ] != '/' ) {
		strcat( path, "/" );  
	}
	strcat( path, di->d_name );

	/* create the new node */
	pl = (PLAYLIST_LAUNCHABLE *)malloc( sizeof( PLAYLIST_LAUNCHABLE ));
	pl->shortname = strdup( di->d_name );
	pl->fullpath = strdup( path );

	/* prepend it to the list */
	pl->next = mikGlob.miklist;
	mikGlob.miklist = pl;
}


/* _scan_playlists_directory
 *	check the passed in directory for any playlist files
 *	if one was found, add it (above)
 */
void mikmodule_scan_playlists_directory( char * dirpath )
{
	char dirp[1024];
	DIR * dir;
	struct dirent * di;

	dir = opendir( dirpath );
	if( dir ) {
		while( (di = readdir( dir )) != NULL ) {
			if( di->d_name && di->d_name[0] ) {
				snprintf( dirp, 1024, "%s%s", dirpath, di->d_name );
				if( mikmodule_ext_tight_playlist( dirp )) {
					mikmodule_scan_add_playlist( dirpath, di );
				}
			}
		}
		closedir( dir );
	}
}


/* _scan_playlists
 * 	scan the various directories through the above function
 */
void mikmodule_scan_playlists( void )
{
	int x=0;

	/* make sure we're starting with a clean list */
	mikGlob.miklist = NULL;

	for( x=0 ; searchpaths[x] != NULL ; x++ ) {
		mikmodule_scan_playlists_directory( searchpaths[x] );
	}
}
